import { __decorate, __metadata, __param, __awaiter } from 'tslib';
import { ɵɵdefineInjectable, Injectable, EventEmitter, Input, Output, Component, forwardRef, NgModule, InjectionToken, Optional, Inject, ɵɵinject, ViewChild, ElementRef, ViewChildren, QueryList, Renderer2, HostListener, Directive, Pipe, ChangeDetectionStrategy } from '@angular/core';
import { NG_VALUE_ACCESSOR, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { assoc, insert, remove, contains, pipe, when, always, isNil, repeat, reverse, range, update, is, sortBy } from 'ramda';
import { CommonModule, DOCUMENT, DatePipe } from '@angular/common';
import { ClrCheckboxModule, ClrDatepickerModule, ClrIconModule, ClrDropdownModule, ClrTooltipModule, ClrSelectModule, ClrRadioModule, ClrInputModule, ClrPasswordModule, ClrTextareaModule } from '@clr/angular';
import { parse as parse$1, format, setHours, getHours, setMinutes, getMinutes } from 'date-fns/esm/fp';
import { BrowserModule } from '@angular/platform-browser';
import { OnChange } from 'property-watch-decorator';
import { cleanupTypeaheadValue, HlcTypeaheadDirectiveModule } from '@ng-holistic/typeahead';
import { BehaviorSubject } from 'rxjs';
import { withLatestFrom } from 'rxjs/operators';

let DictMapperService = class DictMapperService {
    getKey(item) {
        return item['key'];
    }
    getLabel(item) {
        return item['label'];
    }
};
DictMapperService.ɵprov = ɵɵdefineInjectable({ factory: function DictMapperService_Factory() { return new DictMapperService(); }, token: DictMapperService, providedIn: "root" });
DictMapperService = __decorate([
    Injectable({ providedIn: 'root' })
], DictMapperService);
let DictMapperSetterService = class DictMapperSetterService {
    setKey(item, key) {
        // item['key'] = key;
        return assoc('key', key, item);
    }
    setLabel(item, label) {
        // item['label'] = label;
        // return item;
        return assoc('label', label, item);
    }
};
DictMapperSetterService.ɵprov = ɵɵdefineInjectable({ factory: function DictMapperSetterService_Factory() { return new DictMapperSetterService(); }, token: DictMapperSetterService, providedIn: "root" });
DictMapperSetterService = __decorate([
    Injectable({ providedIn: 'root' })
], DictMapperSetterService);

var HlcClrCheckboxesComponent_1;
let HlcClrCheckboxesComponent = HlcClrCheckboxesComponent_1 = class HlcClrCheckboxesComponent {
    constructor(dictMapper) {
        this.dictMapper = dictMapper;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.trackBy = (_, obj) => {
            return this.mapKey(obj);
        };
    }
    ngOnInit() { }
    onChange(item) {
        const key = this.mapKey(item);
        const index = (this.value || []).indexOf(key);
        if (!this.value) {
            this.value = [];
        }
        if (index === -1) {
            this.value = insert(0, key, this.value);
        }
        else {
            this.value = remove(index, 1, this.value);
        }
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    isChecked(item) {
        const itemKey = this.mapKey(item);
        return this.value && contains(itemKey, this.value);
    }
    mapKey(obj) {
        return this.dictMapper.getKey(obj);
    }
    mapLabel(obj) {
        return this.dictMapper.getLabel(obj);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrCheckboxesComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Array)
], HlcClrCheckboxesComponent.prototype, "items", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrCheckboxesComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrCheckboxesComponent.prototype, "readonly", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrCheckboxesComponent.prototype, "valueChange", void 0);
HlcClrCheckboxesComponent = HlcClrCheckboxesComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-checkboxes',
        template: "<clr-checkbox-wrapper *ngFor=\"let item of items; trackBy: trackBy; let index = index\">\r\n    <input\r\n        type=\"checkbox\"\r\n        clrCheckbox\r\n        [checked]=\"isChecked(item)\"\r\n        [value]=\"mapKey(item)\"\r\n        (change)=\"onChange(item)\"\r\n    />\r\n    <label>{{ mapLabel(item) }}</label>\r\n</clr-checkbox-wrapper>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrCheckboxesComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrCheckboxesComponent);

let HlcClrCheckboxesModule = class HlcClrCheckboxesModule {
    constructor() { }
};
HlcClrCheckboxesModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrCheckboxModule],
        declarations: [HlcClrCheckboxesComponent],
        exports: [HlcClrCheckboxesComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrCheckboxesModule);

const DATE_CONVERT_CONFIG = new InjectionToken('DATE_CONVERT_CONFIG');
const parse = (format) => (str) => {
    if (!str) {
        return undefined;
    }
    return parse$1(new Date(), format, str);
};
const ɵ0 = parse;
const translateDateStr = (parseStr, formatStr, str) => {
    if (!str) {
        return undefined;
    }
    return pipe(parse(parseStr), x => x && format(formatStr)(x))(str);
};
const ɵ1 = translateDateStr;
/**
 * Convert dates from controls formatted values to domain formatted strings
 * Input events value -> format to Date -> parse Date to domain formatted string
 */
let DateConvertService = class DateConvertService {
    constructor(config) {
        this.config = config;
    }
    /**
     * Convert control's way formatted date string to date
     */
    parseControlDate(controlValue) {
        return parse(this.domainFormatStr)(controlValue);
    }
    /**
     * Format date to string in control format
     */
    formatToControlsStr(date) {
        if (!date) {
            return undefined;
        }
        return format(this.controlsFormatStr, date);
    }
    /**
     * Convert domain's way formatted date string to date
     */
    parseDomainDate(domainValue) {
        return parse(this.domainFormatStr)(domainValue);
    }
    /**
     * Format date to string in domain format
     */
    formatToDomainStr(date) {
        if (!date) {
            return undefined;
        }
        return format(this.domainFormatStr, date);
    }
    /**
     * Converts control date string to domain string
     */
    convert(controlValue) {
        return translateDateStr(this.controlsFormatStr, this.domainFormatStr, controlValue);
    }
    /**
     * Converts domain string to control date string
     */
    format(str) {
        return translateDateStr(this.domainFormatStr, this.controlsFormatStr, str);
    }
    get domainFormatStr() {
        if (this.config && this.config.domainFormat) {
            return this.config.domainFormat;
        }
        return 'yyyy-MM-dd\'T\'HH:mm:ss';
    }
    get controlsFormatStr() {
        if (this.config && this.config.controlsFormat) {
            return this.config.controlsFormat;
        }
        // TODO: retrieve from ng locale
        return 'MM\/dd\/yyyy';
    }
};
DateConvertService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [DATE_CONVERT_CONFIG,] }] }
];
DateConvertService.ɵprov = ɵɵdefineInjectable({ factory: function DateConvertService_Factory() { return new DateConvertService(ɵɵinject(DATE_CONVERT_CONFIG, 8)); }, token: DateConvertService, providedIn: "root" });
DateConvertService = __decorate([
    Injectable({ providedIn: 'root' }),
    __param(0, Optional()),
    __param(0, Inject(DATE_CONVERT_CONFIG)),
    __metadata("design:paramtypes", [Object])
], DateConvertService);

const DATE_CONFIG = new InjectionToken('DATE_CONFIG');

var HlcClrDateComponent_1;
let HlcClrDateComponent = HlcClrDateComponent_1 = class HlcClrDateComponent {
    constructor(dateConvertService, dateConfig) {
        this.dateConvertService = dateConvertService;
        this.dateConfig = dateConfig;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.formatToDomainStr = (date) => this.dateConvertService.formatToDomainStr(date);
    }
    get date() {
        return this.value ? this.dateConvertService.parseDomainDate(this.value) : undefined;
    }
    set date(val) {
        // control formatted val -> domain date
        this.value = !val
            ? null
            : pipe(
            // if perviouse value not empty take hours + minutes from there
            when(always(this.value), pipe(setHours(getHours(this.value)), setMinutes(getMinutes(this.value)))), this.formatToDomainStr)(val);
        this.onChange();
    }
    get placeholder() {
        return this.dateConfig && this.dateConfig.placeholder;
    }
    ngOnInit() { }
    onReset() {
        this.value = undefined;
        this.onChange();
    }
    onChange() {
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
        if (!this.value) {
            this.resetValue();
        }
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    //
    resetValue() {
        if (this.input) {
            this.input.nativeElement.value = '';
        }
    }
};
HlcClrDateComponent.ctorParameters = () => [
    { type: DateConvertService },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [DATE_CONFIG,] }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateComponent.prototype, "format", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrDateComponent.prototype, "readonly", void 0);
__decorate([
    ViewChild('input', { static: false }),
    __metadata("design:type", ElementRef)
], HlcClrDateComponent.prototype, "input", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrDateComponent.prototype, "valueChange", void 0);
HlcClrDateComponent = HlcClrDateComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-date',
        template: "<input\r\n    clrDate\r\n    [readonly]=\"readonly\"\r\n    [placeholder]=\"placeholder\"\r\n    #input\r\n    type=\"date\"\r\n    [(clrDate)]=\"date\">\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrDateComponent_1),
                multi: true
            }
        ],
        styles: [":host ::ng-deep readonly .datepicker-trigger-icon{display:none}:host ::ng-deep .clr-form-control{margin-top:0}"]
    }),
    __param(1, Optional()), __param(1, Inject(DATE_CONFIG)),
    __metadata("design:paramtypes", [DateConvertService, Object])
], HlcClrDateComponent);

/**
 * Converts domain `range` values to component ones.
 * For example domain range type could be in following formats `[x,y]` or `{from: x, to: y}}
 * We need universal way to convert these values to / from - component / domain
 * By default used `{from, to}` format
 */
let RangeMapperService = class RangeMapperService {
    getRange(item) {
        return [item['from'], item['to']];
    }
    setRange(val) {
        return { from: val[0], to: val[1] };
    }
};
RangeMapperService.ɵprov = ɵɵdefineInjectable({ factory: function RangeMapperService_Factory() { return new RangeMapperService(); }, token: RangeMapperService, providedIn: "root" });
RangeMapperService = __decorate([
    Injectable({ providedIn: 'root' })
], RangeMapperService);

var HlcClrDateRangeComponent_1;
let HlcClrDateRangeComponent = HlcClrDateRangeComponent_1 = class HlcClrDateRangeComponent {
    constructor(rangeMapperService) {
        this.rangeMapperService = rangeMapperService;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
    }
    ngOnInit() { }
    get dateFrom() {
        return this.value && this.rangeMapperService.getRange(this.value)[0];
    }
    get dateTo() {
        return this.value && this.rangeMapperService.getRange(this.value)[1];
    }
    onDateFromChange(val) {
        this.value = this.rangeMapperService.setRange([val, this.dateTo]);
        this.onChange();
    }
    onDateToChange(val) {
        this.value = this.rangeMapperService.setRange([this.dateFrom, val]);
        this.onChange();
    }
    onChange() {
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
        if (!this.value && this.dateComponents) {
            this.dateComponents.forEach(x => x.resetValue());
        }
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrDateRangeComponent.ctorParameters = () => [
    { type: RangeMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateRangeComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateRangeComponent.prototype, "format", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrDateRangeComponent.prototype, "readonly", void 0);
__decorate([
    ViewChildren(HlcClrDateComponent),
    __metadata("design:type", QueryList)
], HlcClrDateRangeComponent.prototype, "dateComponents", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrDateRangeComponent.prototype, "valueChange", void 0);
HlcClrDateRangeComponent = HlcClrDateRangeComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-date-range',
        template: "<hlc-clr-date\r\n    class=\"date-component date-short-component\"\r\n    [value]=\"dateFrom\"\r\n    (valueChange)=\"onDateFromChange($event)\"\r\n    [format]=\"format\"\r\n    [readonly]=\"readonly\"\r\n>\r\n</hlc-clr-date>\r\n-\r\n<hlc-clr-date\r\n    class=\"date-component date-short-component\"\r\n    [value]=\"dateTo\"\r\n    (valueChange)=\"onDateToChange($event)\"\r\n    [format]=\"format\"\r\n    [readonly]=\"readonly\"\r\n>\r\n</hlc-clr-date>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrDateRangeComponent_1),
                multi: true
            }
        ],
        styles: [".date-component{display:inline-table}::ng-deep .date-short-component .date-input{width:120px}"]
    }),
    __metadata("design:paramtypes", [RangeMapperService])
], HlcClrDateRangeComponent);

let HlcClrDateModule = class HlcClrDateModule {
    constructor() { }
};
HlcClrDateModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrDatepickerModule, ClrIconModule],
        declarations: [HlcClrDateComponent],
        exports: [HlcClrDateComponent],
        providers: [DateConvertService],
        entryComponents: [HlcClrDateComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrDateModule);

let HlcClrDateRangeModule = class HlcClrDateRangeModule {
    constructor() { }
};
HlcClrDateRangeModule = __decorate([
    NgModule({
        imports: [CommonModule, HlcClrDateModule],
        declarations: [HlcClrDateRangeComponent],
        exports: [HlcClrDateRangeComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrDateRangeModule);

var HlcClrDateTimeComponent_1;
let HlcClrDateTimeComponent = HlcClrDateTimeComponent_1 = class HlcClrDateTimeComponent {
    constructor(dateConvertService, dateConfig) {
        this.dateConvertService = dateConvertService;
        this.dateConfig = dateConfig;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        //
        this.formatToDomainStr = (date) => this.dateConvertService.formatToDomainStr(date);
        this.hours = Array.from({ length: 24 }, (_, i) => ({
            key: i,
            label: i.toString().padStart(2, '0')
        }));
        this.minutes = Array.from({ length: 60 }, (_, i) => ({
            key: i,
            label: i.toString().padStart(2, '0')
        }));
    }
    get dictMapper() {
        return new DictMapperService();
    }
    get date() {
        return this.dateConvertService.parseDomainDate(this.value);
    }
    set date(val) {
        // control formatted val -> domain date
        this.value = !val
            ? null
            : pipe(
            // if pervious value not empty take hours + minutes from there
            when(always(this.value), pipe(setHours(getHours(this.value)), setMinutes(getMinutes(this.value)))), this.formatToDomainStr)(val);
        this.onChange();
    }
    get placeholder() {
        return this.dateConfig && this.dateConfig.placeholder;
    }
    ngOnInit() { }
    onChange() {
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    //
    get hour() {
        return this.date ? getHours(this.date) : undefined;
    }
    get minute() {
        return this.date ? getMinutes(this.date) : undefined;
    }
    onHourChange(hour) {
        this.value = pipe(setHours(hour), 
        // reset minutes if value still not defined
        when(always(!this.value), setMinutes(0)), this.formatToDomainStr)(this.date || new Date());
        this.onChange();
    }
    onMinuteChange(min) {
        this.value = pipe(setMinutes(min), 
        // reset hours if value still not defined
        when(always(!this.value), setHours(0)), this.formatToDomainStr)(this.date || new Date());
        this.onChange();
    }
};
HlcClrDateTimeComponent.ctorParameters = () => [
    { type: DateConvertService },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [DATE_CONFIG,] }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateTimeComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDateTimeComponent.prototype, "format", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrDateTimeComponent.prototype, "readonly", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrDateTimeComponent.prototype, "valueChange", void 0);
HlcClrDateTimeComponent = HlcClrDateTimeComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-date-time',
        template: "<input [readonly]=\"readonly\" [placeholder]=\"placeholder\" #input type=\"date\" [(clrDate)]=\"date\" />\r\n\r\n<div class=\"time-container\">\r\n    <hlc-clr-select\r\n        [disallowEmpty]=\"true\"\r\n        [items]=\"hours\"\r\n        [value]=\"hour\"\r\n        [dictMapper]=\"dictMapper\"\r\n        (valueChange)=\"onHourChange($event)\"\r\n    ></hlc-clr-select>\r\n\r\n    <hlc-clr-select\r\n        [disallowEmpty]=\"true\"\r\n        [items]=\"minutes\"\r\n        [value]=\"minute\"\r\n        [dictMapper]=\"dictMapper\"\r\n        (valueChange)=\"onMinuteChange($event)\"\r\n    ></hlc-clr-select>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrDateTimeComponent_1),
                multi: true
            }
        ],
        styles: [":host{display:flex}:host ::ng-deep .clr-form-control{margin-top:0}:host ::ng-deep readonly .datepicker-trigger-icon{display:none}.remove__icon{margin-top:10px;margin-left:5px}.hlc-clr-input{width:120px}"]
    }),
    __param(1, Optional()), __param(1, Inject(DATE_CONFIG)),
    __metadata("design:paramtypes", [DateConvertService, Object])
], HlcClrDateTimeComponent);

var HlcClrSelectComponent_1;
let HlcClrSelectComponent = HlcClrSelectComponent_1 = class HlcClrSelectComponent {
    constructor(dictMapperService) {
        this.dictMapperService = dictMapperService;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.trackBy = (_, obj) => {
            return this.mapKey(obj);
        };
    }
    set items(items) {
        this._items = items;
        if (!isNil(this._tmpValue)) {
            this.setValue(this._tmpValue);
        }
    }
    get items() {
        return this._items;
    }
    set value(val) {
        this.setValue(val);
    }
    get value() {
        return this._tmpValue || this._value;
    }
    ngOnInit() { }
    onChange(val) {
        /**
         * Option change event always give string as an option value, event if value actually was other object type,
         * this is wrong obviously when key is number.
         * So use selectedIndex instead of val.target.value
         */
        const selectedIndex = val.target.selectedIndex - (this.disallowEmpty ? 0 : 1);
        this.value = selectedIndex === -1 ? null : this.mapKey((this.items || [])[selectedIndex]);
        this.propagateChange(this.value);
        this.valueChange.emit(this.value);
    }
    mapKey(obj) {
        return (this.dictMapper || this.dictMapperService).getKey(obj);
    }
    mapLabel(obj) {
        return (this.dictMapper || this.dictMapperService).getLabel(obj);
    }
    setValue(val) {
        if (isNil(val)) {
            this._tmpValue = null;
            this._value = null;
            return;
        }
        const hasItem = (this.items || []).some(i => this.mapKey(i) === val);
        if (hasItem) {
            this._tmpValue = null;
            this._value = val;
        }
        else {
            this._tmpValue = val;
            this._value = null;
        }
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrSelectComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], HlcClrSelectComponent.prototype, "items", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], HlcClrSelectComponent.prototype, "value", null);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrSelectComponent.prototype, "naked", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrSelectComponent.prototype, "disallowEmpty", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrSelectComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrSelectComponent.prototype, "dictMapper", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrSelectComponent.prototype, "valueChange", void 0);
HlcClrSelectComponent = HlcClrSelectComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-select',
        template: "<span [class.clr-select-wrapper]=\"!naked\">\r\n    <select\r\n        class=\"hlc-clr-input\"\r\n        [disabled]=\"readonly\"\r\n        (change)=\"onChange($event)\"\r\n        [value]=\"_value\">\r\n        <option\r\n            *ngIf=\"!disallowEmpty\"\r\n            [attr.value]=\"null\"></option>\r\n\r\n        <option\r\n            *ngFor=\"let item of items; trackBy: trackBy\"\r\n            [value]=\"mapKey(item)\" [attr.data]=\"item\">\r\n            {{mapLabel(item)}}\r\n        </option>\r\n    </select>\r\n</span>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrSelectComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrSelectComponent);

let HlcClrSelectModule = class HlcClrSelectModule {
    constructor() { }
};
HlcClrSelectModule = __decorate([
    NgModule({
        imports: [BrowserModule, ClrIconModule, ClrDropdownModule, ReactiveFormsModule],
        declarations: [HlcClrSelectComponent],
        exports: [HlcClrSelectComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrSelectModule);

let HlcClrDateTimeModule = class HlcClrDateTimeModule {
    constructor() { }
};
HlcClrDateTimeModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrDatepickerModule, ClrIconModule, HlcClrSelectModule],
        declarations: [HlcClrDateTimeComponent],
        exports: [HlcClrDateTimeComponent],
        providers: [DateConvertService]
    }),
    __metadata("design:paramtypes", [])
], HlcClrDateTimeModule);

var HlcClrDisplayComponent_1;
let HlcClrDisplayComponent = HlcClrDisplayComponent_1 = class HlcClrDisplayComponent {
    constructor() {
        this.iconClick = new EventEmitter();
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(_) { }
    registerOnTouched(_) { }
    get formattedValue() {
        return this.format ? this.format(this.value) : this.value;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrDisplayComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Function)
], HlcClrDisplayComponent.prototype, "format", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrDisplayComponent.prototype, "icon", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrDisplayComponent.prototype, "iconTooltipText", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrDisplayComponent.prototype, "iconClick", void 0);
HlcClrDisplayComponent = HlcClrDisplayComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-display',
        template: "{{ formattedValue }}\r\n\r\n<clr-tooltip *ngIf=\"iconTooltipText && icon\">\r\n    <button\r\n        clrTooltipTrigger\r\n        class=\"btn btn-link btn-icon btn-sm\"\r\n        type=\"Button\"\r\n        style=\"margin: initial\"\r\n        (click)=\"iconClick.emit(value)\"\r\n    >\r\n        <clr-icon [attr.shape]=\"icon\"></clr-icon>\r\n    </button>\r\n    <clr-tooltip-content clrPosition=\"top-right\" clrSize=\"sm\" *clrIfOpen> <span>Edit</span> </clr-tooltip-content>\r\n</clr-tooltip>\r\n\r\n<button\r\n    *ngIf=\"!iconTooltipText && icon\"\r\n    class=\"btn btn-link btn-icon btn-sm\"\r\n    type=\"Button\"\r\n    style=\"margin: initial\"\r\n    (click)=\"iconClick.emit()\"\r\n>\r\n    <clr-icon [attr.shape]=\"icon\"></clr-icon>\r\n</button>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrDisplayComponent_1),
                multi: true
            }
        ],
        styles: [""]
    })
], HlcClrDisplayComponent);

let HlcClrDisplayModule = class HlcClrDisplayModule {
};
HlcClrDisplayModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrIconModule, ClrTooltipModule],
        declarations: [HlcClrDisplayComponent],
        exports: [HlcClrDisplayComponent]
    })
], HlcClrDisplayModule);

var HlcClrMaskComponent_1;
let HlcClrMaskComponent = HlcClrMaskComponent_1 = class HlcClrMaskComponent {
    constructor() {
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
    }
    ngOnInit() { }
    get textMask() {
        // mask could be set in 2 formats: one full defined mask options and another just mask template string
        // convert all of them to full format here
        return this.mask && (this.mask['mask'] ? this.mask : { mask: this.mask, guide: false });
    }
    onChange($event) {
        if (($event.target.value || '') === (this.value || '')) {
            return;
        }
        this.value = $event.target.value;
        const val = this.unmask ? this.unmask(this.value) : this.value;
        this.propagateChange(val);
        this.valueChange.emit(val);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrMaskComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrMaskComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrMaskComponent.prototype, "mask", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrMaskComponent.prototype, "unmask", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrMaskComponent.prototype, "valueChange", void 0);
HlcClrMaskComponent = HlcClrMaskComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-mask',
        template: "<input\r\n    class=\"clr-input\"\r\n    [placeholder]=\"placeholder || ''\"\r\n    [mask]=\"mask\"\r\n    [value]=\"value || ''\"\r\n    (keyup)=\"onChange($event)\"\r\n    type=\"text\">\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrMaskComponent_1),
                multi: true,
            },
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [])
], HlcClrMaskComponent);

const config = new InjectionToken('config');
const NEW_CONFIG = new InjectionToken('NEW_CONFIG');
const INITIAL_CONFIG = new InjectionToken('INITIAL_CONFIG');
const initialConfig = {
    sufix: '',
    prefix: '',
    clearIfNotMatch: false,
    showTemplate: false,
    showMaskTyped: false,
    dropSpecialCharacters: true,
    specialCharacters: ['/', '(', ')', '.', ':', '-', ' ', '+', ',', '@', '[', ']', '\"', '\''],
    patterns: {
        '0': {
            pattern: new RegExp('\\d'),
        },
        '9': {
            pattern: new RegExp('\\d'),
            optional: true
        },
        'A': {
            pattern: new RegExp('\[a-zA-Z0-9\]')
        },
        'S': {
            pattern: new RegExp('\[a-zA-Z\]')
        },
        'd': {
            pattern: new RegExp('\\d'),
        },
        'm': {
            pattern: new RegExp('\\d'),
        },
    }
};

// tslint:disable
let MaskApplierService = class MaskApplierService {
    constructor(_config) {
        this._config = _config;
        this.maskExpression = '';
        this._shift = new Set();
        this.maskSpecialCharacters = this._config.specialCharacters;
        this.maskAvailablePatterns = this._config.patterns;
        this.clearIfNotMatch = this._config.clearIfNotMatch;
        this.dropSpecialCharacters = this._config.dropSpecialCharacters;
        this.maskSpecialCharacters = this._config.specialCharacters;
        this.maskAvailablePatterns = this._config.patterns;
        this.prefix = this._config.prefix;
        this.sufix = this._config.sufix;
    }
    // tslint:disable-next-line:no-any
    applyMaskWithPattern(inputValue, maskAndPattern) {
        const [mask, customPattern] = maskAndPattern;
        this.customPattern = customPattern;
        return this.applyMask(inputValue, mask);
    }
    applyMask(inputValue, maskExpression, position = 0, cb = () => { }) {
        if (inputValue === undefined || inputValue === null || maskExpression === undefined) {
            return '';
        }
        let cursor = 0;
        let result = ``;
        let multi = false;
        if (inputValue.slice(0, this.prefix.length) === this.prefix) {
            inputValue = inputValue.slice(this.prefix.length, inputValue.length);
        }
        const inputArray = inputValue.toString()
            .split('');
        // tslint:disable-next-line
        for (let i = 0, inputSymbol = inputArray[0]; i
            < inputArray.length; i++, inputSymbol = inputArray[i]) {
            if (cursor === maskExpression.length) {
                break;
            }
            if (this._checkSymbolMask(inputSymbol, maskExpression[cursor]) && maskExpression[cursor + 1] === '?') {
                result += inputSymbol;
                cursor += 2;
            }
            else if (maskExpression[cursor + 1] === '*' && multi
                && this._checkSymbolMask(inputSymbol, maskExpression[cursor + 2])) {
                result += inputSymbol;
                cursor += 3;
                multi = false;
            }
            else if (this._checkSymbolMask(inputSymbol, maskExpression[cursor])
                && maskExpression[cursor + 1]
                    === '*') {
                result += inputSymbol;
                multi = true;
            }
            else if (maskExpression[cursor + 1] === '?' && this._checkSymbolMask(inputSymbol, maskExpression[cursor + 2])) {
                result += inputSymbol;
                cursor += 3;
            }
            else if (this._checkSymbolMask(inputSymbol, maskExpression[cursor])) {
                if (maskExpression[cursor] === 'd') {
                    if (Number(inputSymbol) > 3) {
                        result += 0;
                        cursor += 1;
                        const shiftStep = /\*|\?/g.test(maskExpression.slice(0, cursor))
                            ? inputArray.length
                            : cursor;
                        this._shift.add(shiftStep + this.prefix.length || 0);
                        i--;
                        continue;
                    }
                }
                if (maskExpression[cursor - 1] === 'd') {
                    if (Number(inputValue.slice(cursor - 1, cursor + 1)) > 31) {
                        continue;
                    }
                }
                if (maskExpression[cursor] === 'm') {
                    if (Number(inputSymbol) > 1) {
                        result += 0;
                        cursor += 1;
                        const shiftStep = /\*|\?/g.test(maskExpression.slice(0, cursor))
                            ? inputArray.length
                            : cursor;
                        this._shift.add(shiftStep + this.prefix.length || 0);
                        i--;
                        continue;
                    }
                }
                if (maskExpression[cursor - 1] === 'm') {
                    if (Number(inputValue.slice(cursor - 1, cursor + 1)) > 12) {
                        continue;
                    }
                }
                result += inputSymbol;
                cursor++;
            }
            else if (this.maskSpecialCharacters.indexOf(maskExpression[cursor]) !== -1) {
                result += maskExpression[cursor];
                cursor++;
                const shiftStep = /\*|\?/g.test(maskExpression.slice(0, cursor))
                    ? inputArray.length
                    : cursor;
                this._shift.add(shiftStep + this.prefix.length || 0);
                i--;
            }
            else if (this.maskSpecialCharacters.indexOf(inputSymbol) > -1
                && this.maskAvailablePatterns[maskExpression[cursor]]
                && this.maskAvailablePatterns[maskExpression[cursor]].optional) {
                cursor++;
                i--;
            }
            else if ((this.maskExpression[cursor + 1] === '*')
                && (this._findSpecialChar(this.maskExpression[cursor + 2]))
                && (this._findSpecialChar(inputSymbol) === this.maskExpression[cursor + 2])) {
                cursor += 3;
                result += inputSymbol;
            }
        }
        if (result.length + 1 === maskExpression.length
            && this.maskSpecialCharacters.indexOf(maskExpression[maskExpression.length - 1]) !== -1) {
            result += maskExpression[maskExpression.length - 1];
        }
        let shift = 1;
        let newPosition = position + 1;
        while (this._shift.has(newPosition)) {
            shift++;
            newPosition++;
        }
        cb(this._shift.has(position) ? shift : 0);
        let res = `${this.prefix}${result}`;
        res = this.sufix &&
            cursor === maskExpression.length
            ? `${this.prefix}${result}${this.sufix}`
            : `${this.prefix}${result}`;
        return res;
    }
    _findSpecialChar(inputSymbol) {
        const symbol = this.maskSpecialCharacters
            .find((val) => val === inputSymbol);
        return symbol;
    }
    _checkSymbolMask(inputSymbol, maskSymbol) {
        this.maskAvailablePatterns = this.customPattern
            ? this.customPattern
            : this.maskAvailablePatterns;
        return this.maskAvailablePatterns[maskSymbol]
            && this.maskAvailablePatterns[maskSymbol].pattern
            && this.maskAvailablePatterns[maskSymbol].pattern.test(inputSymbol);
    }
};
MaskApplierService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [config,] }] }
];
MaskApplierService = __decorate([
    Injectable(),
    __param(0, Inject(config)),
    __metadata("design:paramtypes", [Object])
], MaskApplierService);

// tslint:disable
let MaskService = class MaskService extends MaskApplierService {
    constructor(
    // tslint:disable-next-line
    document, _config, _elementRef, _renderer) {
        super(_config);
        this.document = document;
        this._config = _config;
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this.maskExpression = '';
        this.isNumberValue = false;
        this.showMaskTyped = false;
        this.maskIsShown = '';
        // tslint:disable-next-line
        this.onChange = (_) => { };
        this.onTouch = () => { };
        this._formElement = this._elementRef.nativeElement;
    }
    applyMask(inputValue, maskExpression, position = 0, cb = () => { }) {
        this.maskIsShown = this.showMaskTyped
            ? this.maskExpression.replace(/[0-9]/g, '_')
            : '';
        if (!inputValue && this.showMaskTyped) {
            return this.prefix + this.maskIsShown;
        }
        const result = super.applyMask(inputValue, maskExpression, position, cb);
        Array.isArray(this.dropSpecialCharacters)
            ? this.onChange(this._removeMask(this._removeSufix(this._removePrefix(result)), this.dropSpecialCharacters))
            : this.dropSpecialCharacters === true
                ? this.onChange(this.isNumberValue
                    ? Number(this._removeMask(this._removeSufix(this._removePrefix(result)), this.maskSpecialCharacters))
                    : this._removeMask(this._removeSufix(this._removePrefix(result)), this.maskSpecialCharacters))
                : this.onChange(this._removeSufix(this._removePrefix(result)));
        let ifMaskIsShown = '';
        if (!this.showMaskTyped) {
            return result;
        }
        const resLen = result.length;
        const prefNmask = this.prefix + this.maskIsShown;
        ifMaskIsShown = prefNmask.slice(resLen);
        return result + ifMaskIsShown;
    }
    applyValueChanges(position = 0, cb = () => { }) {
        const maskedInput = this.applyMask(this._formElement.value, this.maskExpression, position, cb);
        this._formElement.value = maskedInput;
        if (this._formElement === this.document.activeElement) {
            return;
        }
        this.clearIfNotMatchFn();
    }
    showMaskInInput() {
        if (this.showMaskTyped) {
            this.maskIsShown = this.maskExpression.replace(/[0-9]/g, '_');
        }
    }
    clearIfNotMatchFn() {
        if (this.clearIfNotMatch === true &&
            this.maskExpression.length !== this._formElement.value.length) {
            this.formElementProperty = ['value', ''];
            this.applyMask(this._formElement.value, this.maskExpression);
        }
    }
    set formElementProperty([name, value]) {
        this._renderer.setProperty(this._formElement, name, value);
    }
    _removeMask(value, specialCharactersForRemove) {
        return value
            ? value.replace(this._regExpForRemove(specialCharactersForRemove), '')
            : value;
    }
    _removePrefix(value) {
        if (!this.prefix) {
            return value;
        }
        return value
            ? value.replace(this.prefix, '')
            : value;
    }
    _removeSufix(value) {
        if (!this.sufix) {
            return value;
        }
        return value
            ? value.replace(this.sufix, '')
            : value;
    }
    _regExpForRemove(specialCharactersForRemove) {
        return new RegExp(specialCharactersForRemove.map((item) => `\\${item}`).join('|'), 'gi');
    }
};
MaskService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [config,] }] },
    { type: ElementRef },
    { type: Renderer2 }
];
MaskService.ɵprov = ɵɵdefineInjectable({ factory: function MaskService_Factory() { return new MaskService(ɵɵinject(DOCUMENT), ɵɵinject(config), ɵɵinject(ElementRef), ɵɵinject(Renderer2)); }, token: MaskService, providedIn: "root" });
MaskService = __decorate([
    Injectable({ providedIn: 'root' }),
    __param(0, Inject(DOCUMENT)),
    __param(1, Inject(config)),
    __metadata("design:paramtypes", [Object, Object, ElementRef,
        Renderer2])
], MaskService);

var MaskDirective_1;
// tslint:disable
let MaskDirective = MaskDirective_1 = class MaskDirective {
    constructor(
    // tslint:disable-next-line
    document, _maskService) {
        this.document = document;
        this._maskService = _maskService;
        this._position = null;
        // tslint:disable-next-line
        this.onChange = (_) => { };
        this.onTouch = () => { };
    }
    set maskExpression(value) {
        this._maskValue = value || '';
        if (!this._maskValue) {
            return;
        }
        this._maskService.maskExpression = this._repeatPatternSymbols(this._maskValue);
        this._maskService.formElementProperty = [
            'value',
            this._maskService.applyMask(this._inputValue, this._maskService.maskExpression)
        ];
    }
    set specialCharacters(value) {
        if (!value ||
            !Array.isArray(value) ||
            (Array.isArray(value) && !value.length)) {
            return;
        }
        this._maskService.maskSpecialCharacters = value;
    }
    set patterns(value) {
        if (!value) {
            return;
        }
        this._maskService.maskAvailablePatterns = value;
    }
    set prefix(value) {
        if (!value) {
            return;
        }
        this._maskService.prefix = value;
    }
    set sufix(value) {
        if (!value) {
            return;
        }
        this._maskService.sufix = value;
    }
    set dropSpecialCharacters(value) {
        this._maskService.dropSpecialCharacters = value;
    }
    set showMaskTyped(value) {
        if (!value) {
            return;
        }
        this._maskService.showMaskTyped = value;
    }
    set showTemplate(value) {
        this._maskService.showTemplate = value;
    }
    set clearIfNotMatch(value) {
        this._maskService.clearIfNotMatch = value;
    }
    onInput(e) {
        const el = e.target;
        this._inputValue = el.value;
        if (!this._maskValue) {
            this.onChange(el.value);
            return;
        }
        const position = el.selectionStart === 1
            ? el.selectionStart + this._maskService.prefix.length
            : el.selectionStart;
        let caretShift = 0;
        this._maskService.applyValueChanges(position, (shift) => (caretShift = shift));
        // only set the selection if the element is active
        if (this.document.activeElement !== el) {
            return;
        }
        el.selectionStart = el.selectionEnd =
            this._position !== null
                ? this._position
                : position +
                    // tslint:disable-next-line
                    (e.inputType === 'deleteContentBackward' ? 0 : caretShift);
        this._position = null;
    }
    onBlur() {
        this._maskService.clearIfNotMatchFn();
        this.onTouch();
    }
    onFocus(e) {
        const el = e.target;
        if (el !== null && el.selectionStart !== null &&
            el.selectionStart === el.selectionEnd &&
            el.selectionStart > this._maskService.prefix.length &&
            // tslint:disable-next-line
            e.keyCode !== 38) {
            return;
        }
        if (this._maskService.showMaskTyped) {
            this._maskService.maskIsShown = this._maskService.maskExpression.replace(/[0-9]/g, '_');
        }
        el.value = !el.value || el.value === this._maskService.prefix
            ? this._maskService.prefix + this._maskService.maskIsShown
            : el.value;
        /** fix of cursor position with prefix when mouse click occur */
        if ((el.selectionStart || el.selectionEnd) <= this._maskService.prefix.length) {
            el.selectionStart = this._maskService.prefix.length;
            return;
        }
    }
    a(e) {
        const el = e.target;
        if (e.keyCode === 38) {
            e.preventDefault();
        }
        if (e.keyCode === 37 || e.keyCode === 8) {
            if (el.selectionStart <= this._maskService.prefix.length
                && el.selectionEnd <= this._maskService.prefix.length) {
                e.preventDefault();
            }
            this.onFocus(e);
            if (e.keyCode === 8
                && el.selectionStart === 0
                && el.selectionEnd === el.value.length) {
                el.value = this._maskService.prefix;
                this._position = this._maskService.prefix ? this._maskService.prefix.length : 1;
                this.onInput(e);
            }
        }
    }
    onPaste() {
        this._position = Number.MAX_SAFE_INTEGER;
    }
    /** It writes the value in the input */
    writeValue(inputValue) {
        return __awaiter(this, void 0, void 0, function* () {
            if (inputValue === undefined) {
                inputValue = '';
            }
            if (typeof inputValue === 'number') {
                inputValue = String(inputValue);
                this._maskService.isNumberValue = true;
            }
            inputValue && this._maskService.maskExpression ||
                this._maskService.maskExpression && (this._maskService.prefix || this._maskService.showMaskTyped)
                ? (this._maskService.formElementProperty = [
                    'value',
                    this._maskService.applyMask(inputValue, this._maskService.maskExpression)
                ])
                : (this._maskService.formElementProperty = ['value', inputValue]);
            this._inputValue = inputValue;
        });
    }
    // tslint:disable-next-line
    registerOnChange(fn) {
        this.onChange = fn;
        this._maskService.onChange = this.onChange;
    }
    // tslint:disable-next-line
    registerOnTouched(fn) {
        this.onTouch = fn;
    }
    /** It disables the input element */
    setDisabledState(isDisabled) {
        this._maskService.formElementProperty = ['disabled', isDisabled];
    }
    _repeatPatternSymbols(maskExp) {
        return maskExp.match(/{[0-9]+}/)
            && maskExp.split('')
                .reduce((accum, currval, index) => {
                this._start = (currval === '{') ? index : this._start;
                if (currval !== '}') {
                    return this._maskService._findSpecialChar(currval) ? accum + currval : accum;
                }
                this._end = index;
                const repeatNumber = Number(maskExp
                    .slice(this._start + 1, this._end));
                const repaceWith = new Array(repeatNumber + 1)
                    .join(maskExp[this._start - 1]);
                return accum + repaceWith;
            }, '') || maskExp;
    }
};
MaskDirective.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] }] },
    { type: MaskService }
];
__decorate([
    Input('mask'),
    __metadata("design:type", String),
    __metadata("design:paramtypes", [String])
], MaskDirective.prototype, "maskExpression", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "specialCharacters", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "patterns", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "prefix", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "sufix", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "dropSpecialCharacters", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "showMaskTyped", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "showTemplate", null);
__decorate([
    Input(),
    __metadata("design:type", Object),
    __metadata("design:paramtypes", [Object])
], MaskDirective.prototype, "clearIfNotMatch", null);
__decorate([
    HostListener('input', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [KeyboardEvent]),
    __metadata("design:returntype", void 0)
], MaskDirective.prototype, "onInput", null);
__decorate([
    HostListener('blur'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MaskDirective.prototype, "onBlur", null);
__decorate([
    HostListener('click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], MaskDirective.prototype, "onFocus", null);
__decorate([
    HostListener('keydown', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [KeyboardEvent]),
    __metadata("design:returntype", void 0)
], MaskDirective.prototype, "a", null);
__decorate([
    HostListener('paste'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], MaskDirective.prototype, "onPaste", null);
MaskDirective = MaskDirective_1 = __decorate([
    Directive({
        selector: '[mask]',
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => MaskDirective_1),
                multi: true
            },
            MaskService
        ]
    }),
    __param(0, Inject(DOCUMENT)),
    __metadata("design:paramtypes", [Object, MaskService])
], MaskDirective);

let MaskPipe = class MaskPipe {
    constructor(_maskService) {
        this._maskService = _maskService;
    }
    transform(value, mask) {
        if (!value) {
            return '';
        }
        if (typeof mask === 'string') {
            return this._maskService.applyMask(`${value}`, mask);
        }
        return this._maskService.applyMaskWithPattern(`${value}`, mask);
    }
};
MaskPipe.ctorParameters = () => [
    { type: MaskApplierService }
];
MaskPipe = __decorate([
    Pipe({
        name: 'mask',
        pure: true
    }),
    __metadata("design:paramtypes", [MaskApplierService])
], MaskPipe);

const ɵ0$1 = undefined, ɵ1$1 = initialConfig;
let NgxMaskModule = class NgxMaskModule {
};
NgxMaskModule = __decorate([
    NgModule({
        exports: [MaskDirective, MaskPipe],
        declarations: [MaskDirective, MaskPipe],
        providers: [
            {
                provide: NEW_CONFIG,
                useValue: ɵ0$1
            },
            {
                provide: INITIAL_CONFIG,
                useValue: ɵ1$1
            },
            {
                provide: config,
                useFactory: _configFactory,
                deps: [INITIAL_CONFIG, NEW_CONFIG]
            },
            MaskApplierService
        ]
    })
], NgxMaskModule);
/**
 * @internal
 */
function _configFactory(initConfig, configValue) {
    return typeof configValue === 'function' ? configValue() : Object.assign(Object.assign({}, initConfig), configValue);
}

let HlcClrMaskModule = class HlcClrMaskModule {
};
HlcClrMaskModule = __decorate([
    NgModule({
        imports: [CommonModule, FormsModule, NgxMaskModule],
        declarations: [HlcClrMaskComponent],
        exports: [HlcClrMaskComponent]
    })
], HlcClrMaskModule);

class TextMask {
    static unmaskStrNumber(val) {
        const regex = /[\D|[^.]]+/g;
        return val ? val.replace(regex, '') : null;
    }
    static unmaskNumber(val) {
        const str = TextMask.unmaskStrNumber(val);
        return str ? +str : null;
    }
    static int(n) {
        return repeat('9', n).join('');
    }
    static float(n, i) {
        return repeat('9', n).join('') + '.' + repeat('0', i).join('');
    }
    /*
    static hexColor() {
        const regex = /\d|a|b|c|d|e|f|A|B|C|D|E|F/;
        return ['#', ...R.repeat(regex, 6)];
    }
    */
    static phone() {
        return '+0 (000) 000-0000';
    }
}

const monthYearRequiredValidator = (c) => {
    const val = c.value;
    if (!val || isNil(val.month) || isNil(val.year)) {
        return { required: true };
    }
    else {
        return null;
    }
};

var HlcClrMonthYearSelectComponent_1;
let HlcClrMonthYearSelectComponent = HlcClrMonthYearSelectComponent_1 = class HlcClrMonthYearSelectComponent {
    constructor(datePipe) {
        this.datePipe = datePipe;
        this.valueChange = new EventEmitter();
        //
        this.propagateChange = (_) => { };
    }
    static onYearChange(it) {
        const yfrom = it.yearFrom || 1960;
        const yto = it.yearTo || new Date().getFullYear();
        it.years = reverse(range(yfrom, yto + 1).map(key => ({ key, label: key.toString() })));
        it.months = range(0, 11).map(key => {
            const date = new Date().setMonth(key);
            return {
                key: key + 1,
                label: it.datePipe.transform(date, 'LLLL')
            };
        });
    }
    get yearValue() {
        return this.value && this.value.year;
    }
    get monthValue() {
        return this.value && this.value.month;
    }
    ngOnInit() {
        HlcClrMonthYearSelectComponent_1.onYearChange(this);
    }
    onYearChange(year) {
        this.onValueChanged(assoc('year', year, this.value || {}));
    }
    onMonthChange(month) {
        this.onValueChanged(assoc('month', month, this.value || {}));
    }
    onValueChanged(val) {
        this.value = val;
        this.valueChange.emit(val);
        this.propagateChange(val);
    }
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrMonthYearSelectComponent.ctorParameters = () => [
    { type: DatePipe }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrMonthYearSelectComponent.prototype, "value", void 0);
__decorate([
    OnChange(HlcClrMonthYearSelectComponent_1.onYearChange),
    Input(),
    __metadata("design:type", Number)
], HlcClrMonthYearSelectComponent.prototype, "yearFrom", void 0);
__decorate([
    OnChange(HlcClrMonthYearSelectComponent_1.onYearChange),
    Input(),
    __metadata("design:type", Number)
], HlcClrMonthYearSelectComponent.prototype, "yearTo", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrMonthYearSelectComponent.prototype, "valueChange", void 0);
HlcClrMonthYearSelectComponent = HlcClrMonthYearSelectComponent_1 = __decorate([
    Component({
        selector: 'hlc-month-year-select',
        template: "<hlc-clr-select [items]=\"months\" [value]=\"monthValue\" (valueChange)=\"onMonthChange($event)\"></hlc-clr-select>\r\n<hlc-clr-select [items]=\"years\" [value]=\"yearValue\" (valueChange)=\"onYearChange($event)\"></hlc-clr-select>\r\n",
        providers: [
            DatePipe,
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrMonthYearSelectComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DatePipe])
], HlcClrMonthYearSelectComponent);

let HlcClrMonthYearSelectModule = class HlcClrMonthYearSelectModule {
    constructor() { }
};
HlcClrMonthYearSelectModule = __decorate([
    NgModule({
        imports: [CommonModule, HlcClrSelectModule],
        declarations: [HlcClrMonthYearSelectComponent],
        exports: [HlcClrMonthYearSelectComponent],
        providers: []
    }),
    __metadata("design:paramtypes", [])
], HlcClrMonthYearSelectModule);

var HlcClrMultiSelectComponent_1;
let HlcClrMultiSelectComponent = HlcClrMultiSelectComponent_1 = class HlcClrMultiSelectComponent {
    constructor(dictMapperService) {
        this.dictMapperService = dictMapperService;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.trackBy = (_, obj) => {
            return this.mapKey(obj);
        };
    }
    ngOnInit() { }
    onChange(val) {
        this.value = Array.apply(null, val.target.options)
            .map((opt, i) => [opt, i])
            .filter(([opt]) => opt.selected)
            .map(([_, i]) => this.mapKey(this.items[i]));
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    isSelected(item) {
        const isSelected = contains(this.mapKey(item), this.value || []);
        return isSelected;
    }
    mapKey(obj) {
        return (this.dictMapper || this.dictMapperService).getKey(obj);
    }
    mapLabel(obj) {
        return (this.dictMapper || this.dictMapperService).getLabel(obj);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrMultiSelectComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Array)
], HlcClrMultiSelectComponent.prototype, "items", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrMultiSelectComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrMultiSelectComponent.prototype, "dictMapper", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrMultiSelectComponent.prototype, "valueChange", void 0);
HlcClrMultiSelectComponent = HlcClrMultiSelectComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-multi-select',
        template: "<div class=\"clr-multiselect-wrapper\">\r\n    <select multiple (change)=\"onChange($event)\">\r\n        <option *ngFor=\"let item of items; trackBy: trackBy\" [value]=\"mapKey(item)\" [selected]=\"isSelected(item)\">\r\n            {{ mapLabel(item) }}\r\n        </option>\r\n    </select>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrMultiSelectComponent_1),
                multi: true
            }
        ],
        styles: ["select{width:100%}"]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrMultiSelectComponent);

let HlcClrMultiSelectModule = class HlcClrMultiSelectModule {
    constructor() { }
};
HlcClrMultiSelectModule = __decorate([
    NgModule({
        imports: [CommonModule, ReactiveFormsModule, ClrSelectModule],
        declarations: [HlcClrMultiSelectComponent],
        exports: [HlcClrMultiSelectComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrMultiSelectModule);

var HlcClrOptionsComponent_1;
let HlcClrOptionsComponent = HlcClrOptionsComponent_1 = class HlcClrOptionsComponent {
    constructor(dictMapper) {
        this.dictMapper = dictMapper;
        this.viewType = "simple" /* Simple */;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.trackBy = (_, obj) => {
            return this.mapKey(obj);
        };
    }
    ngOnInit() { }
    onChange(val) {
        if (this.readonly) {
            return;
        }
        this.value = val || null;
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    isChecked(item) {
        return this.mapKey(item) === this.value;
    }
    mapKey(obj) {
        return this.dictMapper.getKey(obj);
    }
    mapLabel(obj) {
        return this.dictMapper.getLabel(obj);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrOptionsComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrOptionsComponent.prototype, "viewType", void 0);
__decorate([
    Input(),
    __metadata("design:type", Array)
], HlcClrOptionsComponent.prototype, "items", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrOptionsComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrOptionsComponent.prototype, "readonly", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrOptionsComponent.prototype, "valueChange", void 0);
HlcClrOptionsComponent = HlcClrOptionsComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-options',
        template: "<ng-container *ngIf=\"viewType === 'simple'; else buttonsView\">\r\n    <clr-radio-wrapper *ngFor=\"let item of items; trackBy: trackBy; let index = index\">\r\n        <input\r\n            [class.disabled]=\"readonly\"\r\n            type=\"radio\"\r\n            clrRadio\r\n            [checked]=\"isChecked(item)\"\r\n            [value]=\"mapKey(item)\"\r\n            (change)=\"onChange($event.target.value)\"\r\n        />\r\n        <label>{{ mapLabel(item) }}</label>\r\n    </clr-radio-wrapper>\r\n</ng-container>\r\n<ng-template #buttonsView>\r\n    <div class=\"btn-group\">\r\n        <div class=\"radio btn\" [class.disabled]=\"readonly\"  *ngFor=\"let item of items; trackBy: trackBy\" (click)=\"onChange(mapKey(item))\">\r\n            <input type=\"radio\" [checked]=\"isChecked(item)\" [value]=\"mapKey(item)\" />\r\n            <label>{{ mapLabel(item) }}</label>\r\n        </div>\r\n    </div>\r\n</ng-template>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrOptionsComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrOptionsComponent);

let HlcClrOptionsModule = class HlcClrOptionsModule {
    constructor() { }
};
HlcClrOptionsModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrRadioModule],
        declarations: [HlcClrOptionsComponent],
        exports: [HlcClrOptionsComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrOptionsModule);

var HlcClrPairsListComponent_1;
let HlcClrPairsListComponent = HlcClrPairsListComponent_1 = class HlcClrPairsListComponent {
    constructor(dictMapper, dictMapperSetter) {
        this.dictMapper = dictMapper;
        this.dictMapperSetter = dictMapperSetter;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.trackBy = (i) => {
            return i;
        };
    }
    ngOnInit() { }
    onChangeKey(itemIndex, key) {
        const value = this.value || [];
        // this.dictMapperSetter.setKey(value[itemIndex], key);
        this.value = update(itemIndex, this.dictMapperSetter.setKey(value[itemIndex], key), value);
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    onChangeValue(itemIndex, label) {
        const value = this.value || [];
        // this.dictMapperSetter.setLabel(value[itemIndex], label);
        this.value = update(itemIndex, this.dictMapperSetter.setLabel(value[itemIndex], label), value);
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    onRemove(itemIndex) {
        const value = this.value || [];
        this.value = remove(itemIndex, 1, value);
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    onAdd() {
        const value = this.value || [];
        const item = pipe((i) => this.dictMapperSetter.setKey(i, ''), (i) => this.dictMapperSetter.setLabel(i, ''))({});
        this.value = insert(0, item, value);
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
    mapKey(obj) {
        return this.dictMapper.getKey(obj);
    }
    mapLabel(obj) {
        return this.dictMapper.getLabel(obj);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrPairsListComponent.ctorParameters = () => [
    { type: DictMapperService },
    { type: DictMapperSetterService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPairsListComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrPairsListComponent.prototype, "readonly", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrPairsListComponent.prototype, "valueChange", void 0);
HlcClrPairsListComponent = HlcClrPairsListComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-pairs-list',
        template: "<button type=\"button\" class=\"btn btn-icon btn-sm btn-secondary\" (click)=\"onAdd()\">\r\n    <clr-icon shape=\"plus\"></clr-icon>Add Item\r\n</button>\r\n<div class=\"clr-row\" *ngFor=\"let item of value; trackBy: trackBy; let index = index\">\r\n    <clr-input-container>\r\n        <input type=\"text\" clrInput [value]=\"mapKey(item)\" (keyup)=\"onChangeKey(index, $event.target.value)\" />\r\n    </clr-input-container>\r\n    <clr-input-container>\r\n        <input type=\"text\" clrInput [value]=\"mapLabel(item)\" (keyup)=\"onChangeValue(index, $event.target.value)\" />\r\n    </clr-input-container>\r\n    <button type=\"button\" class=\"btn btn-icon btn-sm btn-link\" style=\"margin-top: 1rem\" (click)=\"onRemove(index)\">\r\n        <clr-icon shape=\"times\"></clr-icon>\r\n    </button>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrPairsListComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DictMapperService,
        DictMapperSetterService])
], HlcClrPairsListComponent);

let HlcClrPairsListModule = class HlcClrPairsListModule {
    constructor() { }
};
HlcClrPairsListModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrInputModule, ClrIconModule],
        declarations: [HlcClrPairsListComponent],
        exports: [HlcClrPairsListComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrPairsListModule);

var HlcClrPasswordComponent_1;
let HlcClrPasswordComponent = HlcClrPasswordComponent_1 = class HlcClrPasswordComponent {
    constructor() {
        this.propagateChange = (_) => { };
    }
    ngOnInit() { }
    onChange($event) {
        this.value = $event.target.value;
        this.propagateChange($event.target.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    get cleanValue() {
        return isNil(this.value) ? '' : this.value;
    }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrPasswordComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPasswordComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPasswordComponent.prototype, "readonly", void 0);
HlcClrPasswordComponent = HlcClrPasswordComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-password',
        template: "<clr-password-container>\r\n    <input\r\n        class=\"hlc-clr-input\"\r\n        [placeholder]=\"placeholder || ''\"\r\n        [readonly]=\"readonly ? true : undefined\"\r\n        [value]=\"cleanValue\"\r\n        (keyup)=\"onChange($event)\"\r\n        type=\"text\"\r\n        clrPassword\r\n        autocomplete=\"new-password\"\r\n    />\r\n</clr-password-container>\r\n",
        changeDetection: ChangeDetectionStrategy.OnPush,
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrPasswordComponent_1),
                multi: true
            }
        ],
        styles: ["clr-password-container{margin-top:0}"]
    }),
    __metadata("design:paramtypes", [])
], HlcClrPasswordComponent);

let HlcClrPasswordModule = class HlcClrPasswordModule {
};
HlcClrPasswordModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrPasswordModule],
        declarations: [HlcClrPasswordComponent],
        exports: [HlcClrPasswordComponent]
    })
], HlcClrPasswordModule);

const HLC_CLR_PHONE_CONFIG = new InjectionToken('HLC_CLR_PHONE_CONFIG');
const defaultPhoneConfig = {
    useParts: 'CountryCodeNumber',
    getCountry(val) {
        return val && val[0];
    },
    getCode(val) {
        return val && val[1];
    },
    getNumber(val) {
        return val && val[2];
    },
    concatValue(country, code, number) {
        return [country, code, number];
    }
};

var HlcClrPhoneComponent_1;
let HlcClrPhoneComponent = HlcClrPhoneComponent_1 = class HlcClrPhoneComponent {
    constructor(config) {
        this.codeJustFocused = false;
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.config = config || defaultPhoneConfig;
    }
    ngOnInit() { }
    get useCodePart() {
        return this.config.useParts === 'CountryCodeNumber';
    }
    get countryPart() {
        return this.config.getCountry(this.value);
    }
    get codePart() {
        return this.config.getCode && this.config.getCode(this.value);
    }
    get numberPart() {
        return this.config.getNumber(this.value);
    }
    onCountryChange(val) {
        if (val === this.countryPart) {
            return;
        }
        this.value = this.config.concatValue(val, this.codePart, this.numberPart);
        this.onChange();
    }
    onCodeChange(val) {
        if (val === this.codePart) {
            return;
        }
        this.value = this.config.concatValue(this.countryPart, val, this.numberPart);
        this.onChange();
        if (this.codeJustFocused) {
            this.codeJustFocused = false;
            return;
        }
        if (val && val.length === 3) {
            this.numberInput.nativeElement.focus();
        }
    }
    onNumberChange(val) {
        if (val === this.numberPart) {
            return;
        }
        this.value = this.config.concatValue(this.countryPart, this.codePart, val);
        this.onChange();
    }
    //
    onCodeFocus() {
        this.codeJustFocused = true;
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    onChange() {
        this.valueChange.emit(this.value);
        this.propagateChange(this.value);
    }
};
HlcClrPhoneComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [HLC_CLR_PHONE_CONFIG,] }] }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPhoneComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPhoneComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrPhoneComponent.prototype, "readonly", void 0);
__decorate([
    ViewChild('numInput', { static: false }),
    __metadata("design:type", ElementRef)
], HlcClrPhoneComponent.prototype, "numberInput", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrPhoneComponent.prototype, "valueChange", void 0);
HlcClrPhoneComponent = HlcClrPhoneComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-phone',
        template: "<div *ngIf=\"!readonly; else readOnly\">\r\n    +<input\r\n        class=\"clr-input hlc-clr-input small-input\"\r\n        [value]=\"countryPart\"\r\n        (keyup)=\"onCountryChange($event.target.value)\"\r\n        type=\"text\"\r\n        mask=\"999\"\r\n    />\r\n    <ng-container *ngIf=\"useCodePart\">\r\n        (<input\r\n            class=\"clr-input hlc-clr-input small-input\"\r\n            [value]=\"codePart\"\r\n            (keyup)=\"onCodeChange($event.target.value)\"\r\n            type=\"text\"\r\n            mask=\"999\"\r\n            (focus)=\"onCodeFocus()\"\r\n        />)\r\n    </ng-container>\r\n    <input\r\n        #numInput\r\n        class=\"clr-input hlc-clr-input normal-input\"\r\n        [value]=\"numberPart\"\r\n        (keyup)=\"onNumberChange($event.target.value)\"\r\n        type=\"text\"\r\n        mask=\"9999999\"\r\n    />\r\n</div>\r\n<ng-template #readOnly>\r\n    <span class=\"label-readonly\">\r\n        +{{countryPart}} <ng-container *ngIf=\"useCodePart\">({{codePart}})</ng-container> {{numberPart}}\r\n    </span>\r\n</ng-template>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrPhoneComponent_1),
                multi: true
            }
        ],
        styles: [".small-input{width:40px;min-width:40px}.normal-input{width:100px;min-width:100px}.label-readonly{color:#000}"]
    }),
    __param(0, Optional()), __param(0, Inject(HLC_CLR_PHONE_CONFIG)),
    __metadata("design:paramtypes", [Object])
], HlcClrPhoneComponent);

let HlcClrPhoneModule = class HlcClrPhoneModule {
};
HlcClrPhoneModule = __decorate([
    NgModule({
        imports: [CommonModule, NgxMaskModule],
        declarations: [HlcClrPhoneComponent],
        exports: [HlcClrPhoneComponent]
    })
], HlcClrPhoneModule);

let TypeaheadMapperService = class TypeaheadMapperService {
    getKey(item) {
        return item['key'];
    }
    getLabel(item) {
        return item['label'];
    }
    getDescription(item) {
        return item['label'];
    }
};
TypeaheadMapperService.ɵprov = ɵɵdefineInjectable({ factory: function TypeaheadMapperService_Factory() { return new TypeaheadMapperService(); }, token: TypeaheadMapperService, providedIn: "root" });
TypeaheadMapperService = __decorate([
    Injectable({ providedIn: 'root' })
], TypeaheadMapperService);
class TypeaheadStrMapperService {
    getKey(item) {
        return item;
    }
    getLabel(item) {
        return item;
    }
}

var HlcClrTypeaheadComponent_1;
let HlcClrTypeaheadComponent = HlcClrTypeaheadComponent_1 = class HlcClrTypeaheadComponent {
    constructor(mapper) {
        this.mapper = mapper;
        this.valueChange = new EventEmitter();
        this.addNew = new EventEmitter();
        this.propagateChange = (_) => { };
        this.getLabel = (obj) => {
            return this.config && this.config.mapper ? obj[this.config.mapper.getLabel(obj)] : this.mapper.getLabel(obj);
        };
    }
    ngOnInit() { }
    getKey(obj) {
        return this.config && this.config.mapper ? this.config.mapper.getKey(obj) : this.mapper.getKey(obj);
    }
    getDescription(obj) {
        return this.config && this.config.mapper && this.config.mapper.getDescription
            ? this.config.mapper.getDescription(obj)
            : this.mapper.getDescription
                ? this.mapper.getDescription(obj)
                : null;
    }
    get search() {
        return this.config && this.config.search;
    }
    onChange($event) {
        this.value = $event;
        this.propagateChange(this.value);
        this.valueChange.emit(this.value);
    }
    resetValue() {
        this.value = undefined;
        this.input.nativeElement.value = '';
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrTypeaheadComponent.ctorParameters = () => [
    { type: TypeaheadMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "config", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrTypeaheadComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrTypeaheadComponent.prototype, "allowAddNew", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "valueChange", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "addNew", void 0);
__decorate([
    ViewChild('input', { static: false }),
    __metadata("design:type", Object)
], HlcClrTypeaheadComponent.prototype, "input", void 0);
HlcClrTypeaheadComponent = HlcClrTypeaheadComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-typeahead',
        template: "<input\r\n    #input\r\n    type=\"text\"\r\n    [placeholder]=\"placeholder || ''\"\r\n    [hlcTypeahead]=\"search\"\r\n    [value]=\"value\"\r\n    [resultTemplate]=\"result\"\r\n    [resultFormatter]=\"getLabel\"\r\n    [allowAddNew]=\"allowAddNew\"\r\n    (valueChange)=\"onChange($event)\"\r\n    (addNew)=\"addNew.emit($event)\">\r\n<ng-template\r\n    #result\r\n    let-item>\r\n    <span [innerHtml]=\"getLabel(item)\"></span>\r\n    <!--\r\n    <div\r\n        class=\"description\"\r\n        *ngIf=\"getDescription(item)\">\r\n        {{getDescription(item)}}\r\n    </div>\r\n    -->\r\n</ng-template>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrTypeaheadComponent_1),
                multi: true
            }
        ],
        styles: ["::ng-deep .hlc-typeahead-result-item.active{background-color:#1e90ff;color:#fff}::ng-deep .hlc-typeahead-result-item em{font-weight:700}::ng-deep .hlc-typeahead-results-container{background-color:#fff}.description{color:#a9a9a9;font-size:x-small}"]
    }),
    __metadata("design:paramtypes", [TypeaheadMapperService])
], HlcClrTypeaheadComponent);

var HlcClrTagsComponent_1;
let HlcClrTagsComponent = HlcClrTagsComponent_1 = class HlcClrTagsComponent {
    constructor(dictMapper) {
        this.dictMapper = dictMapper;
        this.value$ = new BehaviorSubject([]);
        this.showSorted = true;
        this.addNew = new EventEmitter();
        this.propagateChange = (_) => { };
        this.search = (args$) => {
            return this.config
                .search(args$)
                .pipe(withLatestFrom(this.value$, (items, val) => this.filterItems(val || [])(items)));
        };
        this.filterItems = (excl) => (items) => {
            return items.filter(item => {
                item = cleanupTypeaheadValue(item);
                return !excl.some(i => this.dictMapper.getKey(i) === this.dictMapper.getKey(item));
            });
        };
    }
    set value(val) {
        this.value$.next(val);
    }
    get value() {
        return this.value$.getValue();
    }
    get _config() {
        if (!this.config) {
            return;
        }
        // modify default search, remove already selected tags
        return Object.assign(Object.assign({}, this.config), { search: this.search });
    }
    onChange($event) {
        this.value = [...(this.value || []), $event];
        this.typeahead.resetValue();
        this.propagateChange(this.value);
    }
    onRemove(item) {
        this.value = this.filterItem(item, this.value);
        this.propagateChange(this.value);
    }
    onAddNew(text) {
        this.value = [...(this.value || []), text];
        this.typeahead.resetValue();
        this.propagateChange(this.value);
        this.addNew.emit(text);
    }
    //
    get sortedValue() {
        const sort = is(Function, this.showSorted) ? this.showSorted : this.dictMapper.getLabel.bind(this.dictMapper);
        return this.value && sortBy(sort, this.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    //
    filterItem(item, items) {
        return items.filter(i => this.dictMapper.getKey(i) !== this.dictMapper.getKey(item));
    }
};
HlcClrTagsComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Array),
    __metadata("design:paramtypes", [Array])
], HlcClrTagsComponent.prototype, "value", null);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTagsComponent.prototype, "config", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTagsComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrTagsComponent.prototype, "allowAddNew", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTagsComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTagsComponent.prototype, "showSorted", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrTagsComponent.prototype, "addNew", void 0);
__decorate([
    ViewChild(HlcClrTypeaheadComponent, { static: false }),
    __metadata("design:type", HlcClrTypeaheadComponent)
], HlcClrTagsComponent.prototype, "typeahead", void 0);
HlcClrTagsComponent = HlcClrTagsComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-tags',
        template: "<hlc-clr-tags-list [items]=\"showSorted ? sortedValue : value\" (remove)=\"onRemove($event)\"></hlc-clr-tags-list>\r\n<hlc-clr-typeahead\r\n    #typeahead\r\n    [config]=\"_config\"\r\n    [placeholder]=\"placeholder\"\r\n    [allowAddNew]=\"allowAddNew\"\r\n    (addNew)=\"onAddNew($event)\"\r\n    (valueChange)=\"onChange($event)\"\r\n></hlc-clr-typeahead>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrTagsComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrTagsComponent);

let HlcClrTagsListComponent = class HlcClrTagsListComponent {
    constructor(dictMapper) {
        this.dictMapper = dictMapper;
        this.remove = new EventEmitter();
    }
    ngOnInit() { }
    onRemove(item) {
        this.remove.emit(item);
    }
    trackById(_, item) {
        return this.dictMapper.getKey(item);
    }
    getLabel(item) {
        return this.dictMapper.getLabel(item);
    }
};
HlcClrTagsListComponent.ctorParameters = () => [
    { type: DictMapperService }
];
__decorate([
    Input(),
    __metadata("design:type", Array)
], HlcClrTagsListComponent.prototype, "items", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrTagsListComponent.prototype, "remove", void 0);
HlcClrTagsListComponent = __decorate([
    Component({
        selector: 'hlc-clr-tags-list',
        template: "<ng-container *ngIf=\"items?.length\">\r\n    <div class=\"cat-field__group\">\r\n        <a\r\n            class=\"label label-info clickable tag-item\"\r\n            *ngFor=\"let item of items\"\r\n            (click)=\"onRemove(item)\">\r\n            <clr-icon shape=\"trash\"></clr-icon>\r\n            {{getLabel(item)}}\r\n        </a>\r\n    </div>\r\n</ng-container>\r\n",
        styles: [".cat-field__selector{display:block;margin-top:1rem}.tag-item{margin-bottom:.2rem}"]
    }),
    __metadata("design:paramtypes", [DictMapperService])
], HlcClrTagsListComponent);

let HlcClrTagsListModule = class HlcClrTagsListModule {
    constructor() { }
};
HlcClrTagsListModule = __decorate([
    NgModule({
        imports: [CommonModule, ClrIconModule],
        declarations: [HlcClrTagsListComponent],
        exports: [HlcClrTagsListComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrTagsListModule);

let HlcClrTypeaheadModule = class HlcClrTypeaheadModule {
    constructor() { }
};
HlcClrTypeaheadModule = __decorate([
    NgModule({
        imports: [CommonModule, HlcTypeaheadDirectiveModule],
        declarations: [HlcClrTypeaheadComponent],
        exports: [HlcClrTypeaheadComponent],
    }),
    __metadata("design:paramtypes", [])
], HlcClrTypeaheadModule);

let HlcClrTagsModule = class HlcClrTagsModule {
    constructor() { }
};
HlcClrTagsModule = __decorate([
    NgModule({
        imports: [CommonModule, HlcClrTypeaheadModule, FormsModule, HlcClrTagsListModule],
        declarations: [HlcClrTagsComponent],
        exports: [HlcClrTagsComponent]
    }),
    __metadata("design:paramtypes", [])
], HlcClrTagsModule);

var HlcClrTextAreaComponent_1;
let HlcClrTextAreaComponent = HlcClrTextAreaComponent_1 = class HlcClrTextAreaComponent {
    constructor() {
        this.propagateChange = (_) => { };
    }
    ngOnInit() { }
    onChange($event) {
        if (this.value === undefined && !$event.target.value) {
            // when user set first time focus on empty field don't change null -> ''.
            // Behaviour consistent with text component.
            return;
        }
        this.value = $event.target.value;
        this.propagateChange($event.target.value);
    }
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    get cleanValue() {
        return isNil(this.value) ? '' : this.value;
    }
    get leftCounter() {
        return isNil(this.maxLength) ? null : this.maxLength - (isNil(this.value) ? 0 : this.value.length);
    }
    get counterStyle() {
        if (isNil(this.leftCounter)) {
            return null;
        }
        else if (this.leftCounter < 0) {
            return 'text-counter-danger';
        }
        else if (!isNil(this.warningLimit) && !isNil(this.maxLength) && this.warningLimit > this.leftCounter) {
            return 'text-counter-warning';
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrTextAreaComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrTextAreaComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrTextAreaComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], HlcClrTextAreaComponent.prototype, "rows", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextAreaComponent.prototype, "maxLength", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextAreaComponent.prototype, "warningLimit", void 0);
HlcClrTextAreaComponent = HlcClrTextAreaComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-text-area',
        template: "<div class=\"clr-textarea-wrapper\">\r\n    <textarea\r\n        class=\"clr-textarea\"\r\n        [placeholder]=\"placeholder || ''\"\r\n        [readonly]=\"readonly ? true : undefined\"\r\n        [value]=\"cleanValue\"\r\n        [rows]=\"rows\"\r\n        (keyup)=\"onChange($event)\"\r\n    ></textarea>\r\n    <span *ngIf=\"maxLength\" class=\"text-counter\" [ngClass]=\"counterStyle\">\r\n        <span>{{ maxLength - leftCounter }}</span> / <span>{{maxLength}}</span>\r\n    </span>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrTextAreaComponent_1),
                multi: true
            }
        ],
        styles: ["::ng-deep textarea{border-color:#f6f6f6;border-bottom:1px solid #9a9a9a;background-color:transparent}::ng-deep textarea:focus{box-shadow:none;border-bottom:2px solid #0094d2}::ng-deep textarea:-moz-read-only{border-width:0;box-shadow:none!important}::ng-deep textarea:read-only{border-width:0;box-shadow:none!important}"]
    }),
    __metadata("design:paramtypes", [])
], HlcClrTextAreaComponent);

let HlcClrTextAreaModule = class HlcClrTextAreaModule {
};
HlcClrTextAreaModule = __decorate([
    NgModule({
        imports: [CommonModule, FormsModule, ClrTextareaModule],
        declarations: [HlcClrTextAreaComponent],
        exports: [HlcClrTextAreaComponent],
        entryComponents: [HlcClrTextAreaComponent]
    })
], HlcClrTextAreaModule);

var HlcClrTextComponent_1;
let HlcClrTextComponent = HlcClrTextComponent_1 = class HlcClrTextComponent {
    constructor() {
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
    }
    ngOnInit() { }
    onChange($event) {
        this.value = $event.target.value;
        this.valueChange.emit(this.value);
        this.propagateChange($event.target.value);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
    get cleanValue() {
        return isNil(this.value) ? '' : this.value;
    }
    get leftCounter() {
        return isNil(this.maxLength) ? null : this.maxLength - (isNil(this.value) ? 0 : this.value.length);
    }
    get counterStyle() {
        if (isNil(this.leftCounter)) {
            return null;
        }
        else if (this.leftCounter < 0) {
            return 'text-counter-danger';
        }
        else if (!isNil(this.warningLimit) &&
            !isNil(this.maxLength) &&
            this.warningLimit > this.leftCounter) {
            return 'text-counter-warning';
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrTextComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrTextComponent.prototype, "id", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextComponent.prototype, "placeholder", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextComponent.prototype, "readonly", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextComponent.prototype, "maxLength", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrTextComponent.prototype, "warningLimit", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrTextComponent.prototype, "valueChange", void 0);
HlcClrTextComponent = HlcClrTextComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-text',
        template: "<div class=\"clr-input-wrapper\">\r\n    <input\r\n        class=\"clr-input\"\r\n        [placeholder]=\"placeholder || ''\"\r\n        [readonly]=\"readonly ? true : undefined\"\r\n        [value]=\"cleanValue\"\r\n        (keyup)=\"onChange($event)\"\r\n        type=\"text\"\r\n    />\r\n    <span *ngIf=\"maxLength\" class=\"text-counter\" [ngClass]=\"counterStyle\">\r\n        <span>{{ maxLength - leftCounter }}</span> / <span>{{maxLength}}</span>\r\n    </span>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrTextComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [])
], HlcClrTextComponent);

let HlcClrTextModule = class HlcClrTextModule {
};
HlcClrTextModule = __decorate([
    NgModule({
        imports: [CommonModule, FormsModule, ClrInputModule],
        declarations: [HlcClrTextComponent],
        exports: [HlcClrTextComponent]
    })
], HlcClrTextModule);

var HlcClrToggleComponent_1;
let HlcClrToggleComponent = HlcClrToggleComponent_1 = class HlcClrToggleComponent {
    constructor() {
        this.valueChange = new EventEmitter();
        this.propagateChange = (_) => { };
        this.id = `toggle_${HlcClrToggleComponent_1.togglesCount}`;
        HlcClrToggleComponent_1.togglesCount++;
    }
    ngOnInit() { }
    onChange(val) {
        this.value = val;
        this.propagateChange(val);
        this.valueChange.emit(val);
    }
    //
    writeValue(obj) {
        this.value = obj;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(_) { }
};
HlcClrToggleComponent.togglesCount = 0;
__decorate([
    Input(),
    __metadata("design:type", String)
], HlcClrToggleComponent.prototype, "text", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], HlcClrToggleComponent.prototype, "value", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HlcClrToggleComponent.prototype, "readonly", void 0);
__decorate([
    Output(),
    __metadata("design:type", Object)
], HlcClrToggleComponent.prototype, "valueChange", void 0);
HlcClrToggleComponent = HlcClrToggleComponent_1 = __decorate([
    Component({
        selector: 'hlc-clr-toggle',
        template: "<div class=\"clr-toggle-wrapper\">\r\n    <input type=\"checkbox\" [checked]=\"value ? 'checked' : undefined\" (change)=\"onChange($event.target.checked)\" [attr.id]=\"id\" />\r\n    <label [attr.for]=\"id\">{{ text }}</label>\r\n</div>\r\n",
        providers: [
            {
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => HlcClrToggleComponent_1),
                multi: true
            }
        ],
        styles: [""]
    }),
    __metadata("design:paramtypes", [])
], HlcClrToggleComponent);

let HlcClrToggleModule = class HlcClrToggleModule {
};
HlcClrToggleModule = __decorate([
    NgModule({
        imports: [CommonModule],
        declarations: [HlcClrToggleComponent],
        exports: [HlcClrToggleComponent]
    })
], HlcClrToggleModule);

/**
 * Generated bundle index. Do not edit.
 */

export { DATE_CONFIG, DATE_CONVERT_CONFIG, DateConvertService, DictMapperService, DictMapperSetterService, HLC_CLR_PHONE_CONFIG, HlcClrCheckboxesComponent, HlcClrCheckboxesModule, HlcClrDateComponent, HlcClrDateModule, HlcClrDateRangeComponent, HlcClrDateRangeModule, HlcClrDateTimeComponent, HlcClrDateTimeModule, HlcClrDisplayComponent, HlcClrDisplayModule, HlcClrMaskComponent, HlcClrMaskModule, HlcClrMonthYearSelectComponent, HlcClrMonthYearSelectModule, HlcClrMultiSelectComponent, HlcClrMultiSelectModule, HlcClrOptionsComponent, HlcClrOptionsModule, HlcClrPairsListComponent, HlcClrPairsListModule, HlcClrPasswordComponent, HlcClrPasswordModule, HlcClrPhoneComponent, HlcClrPhoneModule, HlcClrSelectComponent, HlcClrSelectModule, HlcClrTagsComponent, HlcClrTagsModule, HlcClrTextAreaComponent, HlcClrTextAreaModule, HlcClrTextComponent, HlcClrTextModule, HlcClrToggleComponent, HlcClrToggleModule, HlcClrTypeaheadComponent, HlcClrTypeaheadModule, MaskDirective, MaskPipe, MaskService, NgxMaskModule, RangeMapperService, TextMask, TypeaheadMapperService, TypeaheadStrMapperService, _configFactory, defaultPhoneConfig, monthYearRequiredValidator, ɵ0, ɵ1, MaskApplierService as ɵa, config as ɵc, NEW_CONFIG as ɵd, INITIAL_CONFIG as ɵe, initialConfig as ɵf, HlcClrTagsListModule as ɵg, HlcClrTagsListComponent as ɵh };
//# sourceMappingURL=ng-holistic-clr-controls.js.map
