import { InjectionToken } from '@angular/core';
export interface DateTimeConfig {
    controlsFormat?: string;
    domainFormat?: string;
}
export declare const DATE_CONVERT_CONFIG: InjectionToken<DateTimeConfig>;
/**
 * Convert dates from controls formatted values to domain formatted strings
 * Input events value -> format to Date -> parse Date to domain formatted string
 */
export declare class DateConvertService {
    private readonly config;
    constructor(config: DateTimeConfig | undefined);
    /**
     * Convert control's way formatted date string to date
     */
    parseControlDate(controlValue: string | null | undefined): Date | undefined;
    /**
     * Format date to string in control format
     */
    formatToControlsStr(date: Date | null | undefined): string | undefined;
    /**
     * Convert domain's way formatted date string to date
     */
    parseDomainDate(domainValue: string | null | undefined): Date | undefined;
    /**
     * Format date to string in domain format
     */
    formatToDomainStr(date: Date | null | undefined): string | undefined;
    /**
     * Converts control date string to domain string
     */
    convert(controlValue: string | null | undefined): string | undefined;
    /**
     * Converts domain string to control date string
     */
    format(str: string | null | undefined): string | undefined;
    private get domainFormatStr();
    private get controlsFormatStr();
}
